//
// MoonFeature.cpp
// A feature on the moon.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "MoonFeature.h"

// Subclasses we know about: we'll call their file-init routines.
#include "Mare.h"
#include "Crater.h"
#include "Rille.h"
#include "Mountain.h"

#include <MoonCanvas.h>

#include <math.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

// Static variables:
int MoonFeature::sMoonRadius = 1;
int MoonFeature::sCurIndex = 0;
MoonFeature** MoonFeature::sMoonFeatureList = 0;
int MoonFeature::sNumFeatures = 0;

int MoonFeature::sWidth = 0;
int MoonFeature::sHeight = 0;

MoonFeature::MoonFeature() : mName(0)
{
}

MoonFeature::MoonFeature(char* name, char* desc, int size,
               int longitude, int latitude)
{
    Init(name, desc, size, longitude, latitude);
}

void
MoonFeature::Init(char* name, char* desc, int size,
             int longitude, int latitude)
{
    mName = strdup(name);
    if (desc)
        mDesc = strdup(desc);
    else
        mDesc = 0;
    mSize = size;
    mLongitude = longitude;
    mLatitude = latitude;
}

MoonFeature::~MoonFeature()
{
    if (mName) delete[] mName;
    if (mDesc) delete[] mDesc;
}

void MoonFeature::SetCanvasSize(int width, int height)
{
    sWidth = width;
    sHeight = height;
}

char* MoonFeature::GetName()     { return mName; }
char* MoonFeature::GetDesc()     { return mDesc; }
int MoonFeature::GetLongitude()  { return mLongitude; }
int MoonFeature::GetLatitude()   { return mLatitude; }
int MoonFeature::GetSize()       { return mSize; }
int MoonFeature::GetSizeX()      { return mSize; }
int MoonFeature::GetSizeY()      { return mSize; }

#define DEG2KM  (1738 * M_PI / 180.)
// Find the distance from a point to the moonFeature's center.
// If the point is inside the moonFeature, we also set isInside.
double MoonFeature::Distance(double longitude, double latitude, int* isInside)
{
    double dLat = latitude - mLatitude;
    double dLong = longitude - mLongitude;
    if (isInside)
    {
        double sx2 = GetSizeX() / DEG2KM / 2;
        double sy2 = GetSizeY() / DEG2KM / 2;
        *isInside = (longitude > mLongitude - sx2
                     && longitude < mLongitude + sx2
                     && latitude > mLatitude - sy2
                     && latitude < mLatitude + sy2);
    }
    // Distance in km of the point from this feature (moon radius = 1738km):
    return sqrt(dLat*dLat + dLong*dLong) * DEG2KM;
}

void MoonFeature::PaintFeature(MoonCanvas* canvas, int cX, int cY, int radius,
                               int mirrored, int librationE, int librationN)
{
    double longRadians = M_PI/180.* (GetLongitude() + librationE);
    double latRadians = M_PI/180. * (GetLatitude() + librationN);
    double cosLatRadians = cos(latRadians);
    int x = (int)(radius * sin(longRadians) * cosLatRadians);
    int y = (int)(-radius * sin(latRadians));
    double f = radius / 1609.;
    int sizeX = (int)(GetSizeX() * f);
    int sizeY = (int)(GetSizeY() * f);

    canvas->SetColor(255, 255, 100);
    canvas->DrawEllipse(cX + x * mirrored - sizeX/2, cY + y - sizeY/2,
                        sizeX, sizeY);

    LabelName(canvas, cX + x * mirrored - sizeX/2, cY + y - sizeY/2 - 3);
}

// Label the feature's name
void MoonFeature::LabelName(MoonCanvas* canvas, int x, int y)
{
    canvas->DrawString(mName, x, y);
}

// Routines for looping over all known moonFeatures:
void
MoonFeature::InitMoonFeatures(int moonRadius)
{
    sMoonRadius = moonRadius;

    // Delete the old moonFeature list:
    if (sMoonFeatureList)
        DeleteFeatures();
    sMoonFeatureList = new MoonFeature*[2000];

    sNumFeatures = 0;

    Crater::InitCraters();
    Mare::InitMaria();
    Mountain::InitMountains();
    Rille::InitRilles();

    printf("%d features loaded\n", sNumFeatures);
}

void
MoonFeature::DeleteFeatures()
{
    // XXX loop and delete all known features here
    printf("ToDo: delete all known features\n");
}

void MoonFeature::InitLoop()
{
    sCurIndex = -1;
}

MoonFeature* MoonFeature::NextFeature()
{
    if (++sCurIndex >= sNumFeatures)
        return 0;
    return sMoonFeatureList[sCurIndex];
}

