//
// QMoonCanvas.cpp
// Draw the moon and related info.
//
//    Copyright 2002 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

#include "QMoonCanvas.h"

#include "MoonFeature.h"

#include <qpainter.h>
#include <qimage.h>
#include <qmessagebox.h>

// XXX next two should be removable:
#include <stdio.h>
#include <math.h>
#include <stdlib.h>

#define DEGMAX 100
#define DEGMIN  -5

QMoonCanvas::QMoonCanvas( QWidget *parent, const char *name )
    : QWidget( parent, name ), MoonCanvas(),
      mMoonImg(0), mPainter(0)
{
}

QMoonCanvas::~QMoonCanvas()
{
    if (mPainter)
        delete mPainter;
    if (mMoonImg)
        delete mMoonImg;
}

void QMoonCanvas::Clear()
{
    if (sDebugMode)
        printf("QMoonCanvas::Clear\n");
    if (!mPainter)
        return;
    mPainter->setBackgroundColor(QColor("black"));
    mPainter->eraseRect(0, 0, width(), height());
}

void QMoonCanvas::SetColor(int r, int g, int b)
{
    mPainter->setPen(QColor(r, g, b));
}

void QMoonCanvas::DrawLine(int x1, int y1, int x2, int y2)
{
    mPainter->drawLine(x1, y1, x2, y2);
}

void QMoonCanvas::DrawEllipse(int x, int y, int w, int h)
{
    mPainter->drawEllipse(x, y, w, h);
}

void QMoonCanvas::DrawString(char* str, int x, int y)
{
    int w = width();
    int h = height();

    // Opaque mode doesn't work on the Zaurus, so don't bother:
    //painter->setBackgroundMode(QPainter::OpaqueMode);
    QString name (str);

    // Shift the label if it would go offscreen:
    QRect bounds = mPainter->boundingRect(0, 0, w, h,
                                          Qt::AlignLeft, name, -1);
    if (x + bounds.right() >= w)
        x = w - bounds.right() - 4;
    if (y < bounds.bottom())
        y = bounds.bottom() + 1;

    mPainter->drawText(x, y, name);
}

void QMoonCanvas::Zoom(int level)
{
    if (level > 1) level = 1;
    if (level < 0) level = 0;
    mZoomLevel = level;

    if (mMoonImg)
        delete mMoonImg;
    mMoonImg = 0;

    if (mZoomLevel > 0)
    {
        QString dir(DATADIR);
        QString mirror("-mirror");
        QString nomirror("");
        QString extension(".png");
        QString quadstr;
        int quad;
        // Figure out where we are from our object list:
        if (mCraterNum <= 0)
            quad = 1;
        else
        {
            int longitude = mMatches[0]->GetLongitude();
            int latitude = mMatches[0]->GetLatitude();
            if (longitude < 0 && latitude > 0)
                quad = 1;
            else if (longitude > 0 && latitude > 0)
                quad = 2;
            else if (longitude < 0 && latitude < 0)
                quad = 3;
            else 
                quad = 4;
        }

        // Open the image:
        QString moonImgStr = dir + "quad" + QString::number(quad)
                              + (mMirrored < 0 ? mirror : nomirror)
                              + extension;
        mMoonImg = new QImage(moonImgStr);
        if (sDebugMode)
            printf("Opened %s: %d x %d\n",
                   (const char*)moonImgStr,
                   mMoonImg->width(), mMoonImg->height());
        mMoonRadius = (int)(.972 * mMoonImg->width());
        mImgLibrationE = 4;
        mImgLibrationN = 1;

        // Fine-tune the offsets for these images (what a pain):
        switch (quad)
        {
          case 2:
              mMinLong = DEGMIN;
              mMaxLong = DEGMAX;
              mMinLat = DEGMIN;
              mMaxLat = DEGMAX;
              quadstr = "quad2";
              // The moon center numbers are specific to these images.
              mMoonCenterY = (int)(.976*mMoonImg->height());
              if (mMirrored < 0)
                  mMoonCenterX = (int)(.963*mMoonImg->width());
              else
                  mMoonCenterX = (int)(.040*mMoonImg->width());
              break;
          case 3:
              mMinLong = -DEGMAX;
              mMaxLong = -DEGMIN;
              mMinLat = -DEGMIN;
              mMaxLat = -DEGMAX;
              quadstr = "quad3";
              mMoonCenterY = (int)(.046*mMoonImg->height());
              if (mMirrored < 0)
                  mMoonCenterX = (int)(.016*mMoonImg->width());
              else
                  mMoonCenterX = (int)(.986*mMoonImg->width());
              break;
          case 4:
              mMinLong = DEGMAX;
              mMaxLong = DEGMIN;
              mMinLat = -DEGMIN;
              mMaxLat = -DEGMAX;
              quadstr = "quad4";
              mMoonCenterY = (int)(.033*mMoonImg->height());
              if (mMirrored < 0)
                  mMoonCenterX = (int)(.958*mMoonImg->width());
              else
                  mMoonCenterX = (int)(.041*mMoonImg->width());
              break;
          case 0:
          case 1:
          default:
              mMinLong = -DEGMAX;
              mMaxLong = -DEGMIN;
              mMinLat = DEGMIN;
              mMaxLat = DEGMAX;
              quadstr = "quad1";
              mMoonCenterY = (int)(.980*mMoonImg->height());
              if (mMirrored < 0)
                  mMoonCenterX = (int)(.023*mMoonImg->width());
              else
                  mMoonCenterX = (int)(.976*mMoonImg->width());
              break;
        }

        if (mMoonRadius > 1)
            return;

        // If we fell through to here, there's something wrong with
        // the image.  So we'll just fall back to zoom level 1.
        if (mMoonImg)
            delete mMoonImg;
    }

    // Zoom level 1 (full moon):
    mMinLong = mMinLat = -DEGMAX;
    mMaxLong = mMaxLat = DEGMAX;
    mImgLibrationE = 5;
    mImgLibrationN = 0;
    if (mMirrored < 0) {
        mMoonImg = new QImage
            (QString(DATADIR "fullmoon-mirror.png"));
        if (sDebugMode)
            printf("Opened fullmoon-mirror.png: %d x %d\n",
                   mMoonImg->width(), mMoonImg->height());
    } else {
        mMoonImg = new QImage
            (QString(DATADIR "fullmoon.png"));
        if (sDebugMode)
            printf("Opened fullmoon.png: %d x %d\n",
                   mMoonImg->width(), mMoonImg->height());
    }

    setMinimumWidth(mMoonImg->width());
    setMinimumHeight(mMoonImg->height());
    if (sDebugMode)
        printf("Setting min size to %d x %d\n",
               mMoonImg->width(), mMoonImg->height());

    mMoonCenterX = mMoonImg->width()/2;
    mMoonCenterY = mMoonImg->height()/2;
    if (sDebugMode)
        printf("Center at (%d, %d)\n", mMoonCenterX, mMoonCenterY);

    mMoonRadius = mMoonImg->width()/2 - 2;
}

int QMoonCanvas::Width()
{
    return width();
}

int QMoonCanvas::Height()
{
    return height();
}

void QMoonCanvas::paintEvent(QPaintEvent*)
{
    //printf("QMoonCanvas::paintEvent\n");
    Paint();
}

void QMoonCanvas::Paint()
{
    //printf("QMoonCanvas::Paint\n");
    if (mMoonRadius <= 0)
        Init();

    if (!mPainter)
        mPainter = new QPainter(this);

    Clear();
    mPainter->setRasterOp(Qt::CopyROP);
    mPainter->drawImage(0, 0, *mMoonImg);

    PaintFeatures();

    delete mPainter;
    mPainter = 0;

    return;
}

void QMoonCanvas::PaintFeature(MoonFeature* feature)
{
    if (!mPainter)
        mPainter = new QPainter(this);
    feature->PaintFeature(this, mMoonCenterX, mMoonCenterY, mMoonRadius,
                          mMirrored, mImgLibrationE, mImgLibrationN);

    mDescriptions.append(feature->GetName());
    QString numstr;
    numstr.setNum(feature->GetSize());
    mDescriptions.append(" (");
    mDescriptions.append(numstr);
    mDescriptions.append(" km)\n");
    mDescriptions.append(feature->GetDesc());
    mDescriptions.append("\n");
}

void QMoonCanvas::mousePressEvent(QMouseEvent *event)
{
    MousePress(event->pos().x(), event->pos().y(), 1,
               event->type() == QEvent::MouseButtonDblClick ? 2 : 1);

    // Remember the time:
    gettimeofday(&mLastButtonDownTime, 0);

    // We're only interested in doubleclick events:
    if (event->type() != QEvent::MouseButtonDblClick) {
        mCraterNum = 0;
        mDescriptions.truncate(0);
    }
    else
        QMessageBox::information(this, "QMoon", mDescriptions,
                                 QMessageBox::Ok);
}

void QMoonCanvas::mouseReleaseEvent(QMouseEvent *event)
{
    MouseRelease(event->pos().x(), event->pos().y(), 1,
                 event->type() == QEvent::MouseButtonDblClick ? 2 : 1);
}

